@@ -63,7 +63,7 @@ void ConnectionValidator::checkServerAndAuth()
         // We want to reset the QNAM proxy so that the global proxy settings are used (via ClientProxy settings)
         _account->networkAccessManager()->setProxy(QNetworkProxy(QNetworkProxy::DefaultProxy));
         // use a queued invocation so we're as asynchronous as with the other code path
-        QMetaObject::invokeMethod(this, "slotCheckServerAndAuth", Qt::QueuedConnection);
+        QMetaObject::invokeMethod(this, "slotCheckRedirectCostFreeUrl", Qt::QueuedConnection);
     }
 }
 
@@ -81,10 +81,21 @@ void ConnectionValidator::systemProxyLookupDone(const QNetworkProxy &proxy)
     }
     _account->networkAccessManager()->setProxy(proxy);
 
-    slotCheckServerAndAuth();
+    slotCheckRedirectCostFreeUrl();
 }
 
 // The actual check
+
+void ConnectionValidator::slotCheckRedirectCostFreeUrl()
+{
+    const auto checkJob = new CheckRedirectCostFreeUrlJob(_account, this);
+    checkJob->setTimeout(timeoutToUseMsec);
+    checkJob->setIgnoreCredentialFailure(true);
+    connect(checkJob, &CheckRedirectCostFreeUrlJob::timeout, this, &ConnectionValidator::slotJobTimeout);
+    connect(checkJob, &CheckRedirectCostFreeUrlJob::jobFinished, this, &ConnectionValidator::slotCheckRedirectCostFreeUrlFinished);
+    checkJob->start();
+}
+
 void ConnectionValidator::slotCheckServerAndAuth()
 {
     auto *checkJob = new CheckServerJob(_account, this);
@@ -96,6 +107,15 @@ void ConnectionValidator::slotCheckServerAndAuth()
     checkJob->start();
 }
 
+void ConnectionValidator::slotCheckRedirectCostFreeUrlFinished(int statusCode)
+{
+    if (statusCode >= 301 && statusCode <= 307) {
+        reportResult(StatusRedirect);
+        return;
+    }
+    slotCheckServerAndAuth();
+}
+
 void ConnectionValidator::slotStatusFound(const QUrl &url, const QJsonObject &info)
 {
     // Newer servers don't disclose any version in status.php anymore
