@@ -82,8 +82,9 @@ public:
      * Version 2: introduction of metadata_parent hash in 2.6.0
      *            (version remains readable by 2.5.1)
      * Version 3: introduction of new windows vfs mode in 2.6.0
+     * Version 5: available in oC client 4.0.0 and 4.2.0
      */
-    static int maxSettingsVersion() { return 3; }
+    static int maxSettingsVersion() { return 5; }
 
     /// Ensure / as separator and trailing /.
     static QString prepareLocalPath(const QString &path);
@@ -233,6 +234,9 @@ public:
     /// Removes the folder from the account's settings.
     void removeFromSettings() const;
 
+    /* Check if the path is ignored. */
+    [[nodiscard]] bool pathIsIgnored(const QString &path) const;
+
     /**
       * Returns whether a file inside this folder should be excluded.
       */
@@ -298,6 +302,10 @@ public:
 
     QString fileFromLocalPath(const QString &localPath) const;
 
+    void whitelistPath(const QString &path);
+    void blacklistPath(const QString &path);
+    void migrateBlackListPath(const QString &legacyPath);
+
 signals:
     void syncStateChange();
     void syncStarted();
@@ -347,6 +355,13 @@ public slots:
     */
     void slotFilesLockReleased(const QSet<QString> &files);
 
+    /*
+     * Triggered when lock files were added
+     */
+    void slotFilesLockImposed(const QSet<QString> &files);
+
+    void slotLockedFilesFound(const QSet<QString> &files);
+
     /**
      * Mark a virtual file as being requested for download, and start a sync.
      *
@@ -402,6 +417,7 @@ private slots:
     void slotEmitFinishedDelayed();
 
     void slotNewBigFolderDiscovered(const QString &, bool isExternal);
+    void slotExistingFolderNowBig(const QString &folderPath);
 
     void slotLogPropagationStart();
 
@@ -464,6 +480,12 @@ private:
 
     void correctPlaceholderFiles();
 
+    void appendPathToSelectiveSyncList(const QString &path, const SyncJournalDb::SelectiveSyncListType listType);
+    void removePathFromSelectiveSyncList(const QString &path, const SyncJournalDb::SelectiveSyncListType listType);
+
+    static void postExistingFolderNowBigNotification(const QString &folderPath);
+    void postExistingFolderNowBigActivity(const QString &folderPath) const;
+
     AccountStatePtr _accountState;
     FolderDefinition _definition;
     QString _canonicalLocalPath; // As returned with QFileInfo:canonicalFilePath.  Always ends with "/"
@@ -543,6 +565,8 @@ private:
 
     QMetaObject::Connection _officeFileLockReleaseUnlockSuccess;
     QMetaObject::Connection _officeFileLockReleaseUnlockFailure;
+    QMetaObject::Connection _fileLockSuccess;
+    QMetaObject::Connection _fileLockFailure;
 };
 }
 
