@@ -27,8 +27,7 @@
 #include <QScopedPointer>
 #include <QSet>
 #include <QDir>
-
-class QTimer;
+#include <QTimer>
 
 namespace OCC {
 
@@ -50,6 +49,13 @@ class Folder;
 class FolderWatcher : public QObject
 {
     Q_OBJECT
+
+    struct FileLockingInfo {
+        enum class Type { Unset = -1, Locked, Unlocked };
+        QString path;
+        Type type = Type::Unset;
+    };
+
 public:
     // Construct, connect signals, call init()
     explicit FolderWatcher(Folder *folder = nullptr);
@@ -60,9 +66,6 @@ public:
      */
     void init(const QString &root);
 
-    /* Check if the path is ignored. */
-    bool pathIsIgnored(const QString &path);
-
     /**
      * Returns false if the folder watcher can't be trusted to capture all
      * notifications.
@@ -93,6 +96,15 @@ signals:
     */
     void filesLockReleased(const QSet<QString> &files);
 
+    /*
+     * Emitted when lock files were added
+     */
+    void filesLockImposed(const QSet<QString> &files);
+
+    void lockFilesFound(const QSet<QString> &files);
+
+    void lockedFilesFound(const QSet<QString> &files);
+
     /**
      * Emitted if some notifications were lost.
      *
@@ -117,6 +129,7 @@ protected slots:
 
 private slots:
     void startNotificationTestWhenReady();
+    void lockChangeDebouncingTimerTimedOut();
 
 protected:
     QHash<QString, int> _pendingPathes;
@@ -132,12 +145,22 @@ private:
 
     void appendSubPaths(QDir dir, QStringList& subPaths);
 
-    QString possiblyAddUnlockedFilePath(const QString &path);
+    [[nodiscard]] FileLockingInfo lockFileTargetFilePath(const QString &path, const QString &lockFileNamePattern) const;
+    [[nodiscard]] QString findMatchingUnlockedFileInDir(const QString &dirPath, const QString &lockFileName) const;
+
     QString findMatchingUnlockedFileInDir(const QString &dirPath, const QString &lockFileName);
 
+    /* Check if the path should be igored by the FolderWatcher. */
+    [[nodiscard]] bool pathIsIgnored(const QString &path) const;
+
     /** Path of the expected test notification */
     QString _testNotificationPath;
 
+    QSet<QString> _unlockedFiles;
+    QSet<QString> _lockedFiles;
+
+    QTimer _lockChangeDebouncingTimer;
+
     friend class FolderWatcherPrivate;
 };
 }
