@@ -71,14 +71,14 @@ public:
      * Constructor for shares
      */
     explicit Share(AccountPtr account,
-        const QString &id,
-        const QString &owner,
-        const QString &ownerDisplayName,
-        const QString &path,
-        const ShareType shareType,
-        bool isPasswordSet = false,
-        const Permissions permissions = SharePermissionDefault,
-        const ShareePtr shareWith = ShareePtr(nullptr));
+                   const QString &id,
+                   const QString &owner,
+                   const QString &ownerDisplayName,
+                   const QString &path,
+                   const ShareType shareType,
+                   bool isPasswordSet = false,
+                   const Permissions permissions = SharePermissionAll,
+                   const ShareePtr shareWith = ShareePtr(nullptr));
 
     /**
      * The account the share is defined on.
@@ -387,7 +387,7 @@ public:
      * @param password The password of the share, may be empty
      *
      * On success the signal linkShareCreated is emitted
-     * For older server the linkShareRequiresPassword signal is emitted when it seems appropiate
+     * For older server the linkShareRequiresPassword signal is emitted when it seems appropriate
      * In case of a server error the serverError signal is emitted
      */
     void createLinkShare(const QString &path,
