@@ -44,30 +44,47 @@ bool SortedActivityListModel::lessThan(const QModelIndex &sourceLeft, const QMod
         return false;
     }
 
+    // Let's now check for errors as we want those near the top too
+    // Sync result errors go first
+    const auto leftSyncResultStatus = leftActivity._syncResultStatus;
+    const auto rightSyncResultStatus = rightActivity._syncResultStatus;
+
+    const auto leftIsSyncResultError = leftSyncResultStatus == SyncResult::Error ||
+                                       leftSyncResultStatus == SyncResult::SetupError ||
+                                       leftSyncResultStatus == SyncResult::Problem;
+
+    const auto rightIsSyncResultError = rightSyncResultStatus == SyncResult::Error ||
+                                        rightSyncResultStatus == SyncResult::SetupError ||
+                                        rightSyncResultStatus == SyncResult::Problem;
+
+    if (leftIsSyncResultError != rightIsSyncResultError) {
+        return leftIsSyncResultError;
+    } // If they are both errors then we will order the errors according to enum order later
+
+    // Then sync file item status errors
+    const auto leftSyncFileItemStatus = leftActivity._syncFileItemStatus;
+    const auto rightSyncFileItemStatus = rightActivity._syncFileItemStatus;
+    const bool leftIsErrorFileItemStatus = leftSyncFileItemStatus != SyncFileItem::NoStatus &&
+                                           leftSyncFileItemStatus != SyncFileItem::Success;
+
+    const bool rightIsErrorFileItemStatus = rightSyncFileItemStatus != SyncFileItem::NoStatus &&
+                                            rightSyncFileItemStatus != SyncFileItem::Success;
+
+    if (leftIsErrorFileItemStatus != rightIsErrorFileItemStatus) {
+        return leftIsErrorFileItemStatus;
+    }
+
+    // Let's go back to more broadly comparing by type
     if (const auto rightType = rightActivity._type; leftType != rightType) {
         return leftType < rightType;
     }
 
-    const auto leftSyncFileItemStatus = leftActivity._syncFileItemStatus;
-    const auto rightSyncFileItemStatus = rightActivity._syncFileItemStatus;
-
-    // Then compare by status
-    if (leftSyncFileItemStatus != rightSyncFileItemStatus) {
-        // We want to shove erors towards the top.
-        return (leftSyncFileItemStatus != SyncFileItem::NoStatus &&
-                leftSyncFileItemStatus != SyncFileItem::Success) ||
-                leftSyncFileItemStatus == SyncFileItem::FatalError ||
-                leftSyncFileItemStatus < rightSyncFileItemStatus;
+    if (leftSyncResultStatus != rightSyncResultStatus) {
+        return leftSyncResultStatus < rightSyncResultStatus;
     }
 
-    const auto leftSyncResultStatus = leftActivity._syncResultStatus;
-    const auto rightSyncResultStatus = rightActivity._syncResultStatus;
-
-    if (leftSyncResultStatus != rightSyncResultStatus) {
-        // We only ever use SyncResult::Error in activities
-        return (leftSyncResultStatus != SyncResult::Undefined &&
-                leftSyncResultStatus != SyncResult::Success) ||
-                leftSyncResultStatus == SyncResult::Error;
+    if (leftSyncFileItemStatus != rightSyncFileItemStatus) {
+        return leftSyncFileItemStatus < rightSyncFileItemStatus;
     }
 
     // Finally sort by time, latest first
