@@ -35,16 +35,23 @@
 
 namespace OCC {
 
-QUrl PropagateUploadFileNG::chunkUrl(int chunk)
+constexpr auto relativeUploadsPath = "remote.php/dav/uploads/";
+
+QUrl PropagateUploadFileNG::chunkUploadFolderUrl() const
 {
-    QString path = QLatin1String("remote.php/dav/uploads/")
-        + propagator()->account()->davUser()
-        + QLatin1Char('/') + QString::number(_transferId);
-    if (chunk >= 0) {
-        // We need to do add leading 0 because the server orders the chunk alphabetically
-        path += QLatin1Char('/') + QString::number(chunk).rightJustified(16, '0'); // 1e16 is 10 petabyte
-    }
-    return Utility::concatUrlPath(propagator()->account()->url(), path);
+    const QString userUploadPath = relativeUploadsPath + propagator()->account()->davUser();
+    const QString chunksUploadPath = userUploadPath + QLatin1Char('/') + QString::number(_transferId);
+    return Utility::concatUrlPath(propagator()->account()->url(), chunksUploadPath);
+}
+
+QUrl PropagateUploadFileNG::chunkUrl(const int chunk) const
+{
+    Q_ASSERT(chunk >= 1);
+    constexpr auto maxChunkDigits = 5; // Chunk V2: max num of chunks is 10000
+
+    // We need to do add leading 0 because the server orders the chunk alphabetically
+    const auto chunkNumString = QString("%1").arg(chunk, maxChunkDigits, 10, QChar('0'));
+    return Utility::concatUrlPath(chunkUploadFolderUrl(), chunkNumString);
 }
 
 /*
@@ -78,6 +85,14 @@ QUrl PropagateUploadFileNG::chunkUrl(int chunk)
 
  */
 
+QByteArray PropagateUploadFileNG::destinationHeader() const
+{
+    const auto davUrl = Utility::trailingSlashPath(propagator()->account()->davUrl().toString());
+    const auto remotePath = Utility::noLeadingSlashPath(propagator()->fullRemotePath(_fileToUpload._file));
+    const auto destination = QString(davUrl + remotePath);
+    return destination.toUtf8();
+}
+
 void PropagateUploadFileNG::doStartUpload()
 {
     propagator()->_activeJobList.append(this);
@@ -87,11 +102,10 @@ void PropagateUploadFileNG::doStartUpload()
     if (_item->_modtime <= 0) {
         qCWarning(lcPropagateUpload()) << "invalid modified time" << _item->_file << _item->_modtime;
     }
-    if (progressInfo._valid && progressInfo.isChunked() && progressInfo._modtime == _item->_modtime
-            && progressInfo._size == _item->_size) {
+    if (progressInfo._valid && progressInfo.isChunked() && progressInfo._modtime == _item->_modtime && progressInfo._size == _item->_size) {
         _transferId = progressInfo._transferid;
-        auto url = chunkUrl();
-        auto job = new LsColJob(propagator()->account(), url, this);
+
+        const auto job = new LsColJob(propagator()->account(), chunkUploadFolderUrl(), this);
         _jobs.append(job);
         job->setProperties(QList<QByteArray>() << "resourcetype"
                                                << "getcontentlength");
@@ -107,7 +121,7 @@ void PropagateUploadFileNG::doStartUpload()
         // The upload info is stale. remove the stale chunks on the server
         _transferId = progressInfo._transferid;
         // Fire and forget. Any error will be ignored.
-        (new DeleteJob(propagator()->account(), chunkUrl(), this))->start();
+        (new DeleteJob(propagator()->account(), chunkUploadFolderUrl(), this))->start();
         // startNewUpload will reset the _transferId and the UploadInfo in the db.
     }
 
@@ -116,12 +130,14 @@ void PropagateUploadFileNG::doStartUpload()
 
 void PropagateUploadFileNG::slotPropfindIterate(const QString &name, const QMap<QString, QString> &properties)
 {
-    if (name == chunkUrl().path()) {
+    if (name == chunkUploadFolderUrl().path()) {
         return; // skip the info about the path itself
     }
+
     bool ok = false;
     QString chunkName = name.mid(name.lastIndexOf('/') + 1);
     auto chunkId = chunkName.toLongLong(&ok);
+
     if (ok) {
         ServerChunkInfo chunkinfo = { properties["getcontentlength"].toLongLong(), chunkName };
         _serverChunks[chunkId] = chunkinfo;
@@ -134,7 +150,8 @@ void PropagateUploadFileNG::slotPropfindFinished()
     slotJobDestroyed(job); // remove it from the _jobs list
     propagator()->_activeJobList.removeOne(this);
 
-    _currentChunk = 0;
+    // Chunked upload v2: numbers range from 1 to 10000
+    _currentChunk = 1;
     _sent = 0;
     while (_serverChunks.contains(_currentChunk)) {
         _sent += _serverChunks[_currentChunk].size;
@@ -151,7 +168,7 @@ void PropagateUploadFileNG::slotPropfindFinished()
 
         // Wipe the old chunking data.
         // Fire and forget. Any error will be ignored.
-        (new DeleteJob(propagator()->account(), chunkUrl(), this))->start();
+        (new DeleteJob(propagator()->account(), chunkUploadFolderUrl(), this))->start();
 
         propagator()->_activeJobList.append(this);
         startNewUpload();
@@ -169,7 +186,7 @@ void PropagateUploadFileNG::slotPropfindFinished()
         // we should remove the later chunks. Otherwise when we do dynamic chunk sizing, we may end up
         // with corruptions if there are too many chunks, or if we abort and there are still stale chunks.
         for (const auto &serverChunk : qAsConst(_serverChunks)) {
-            auto job = new DeleteJob(propagator()->account(), Utility::concatUrlPath(chunkUrl(), serverChunk.originalName), this);
+            auto job = new DeleteJob(propagator()->account(), Utility::concatUrlPath(chunkUploadFolderUrl(), serverChunk.originalName), this);
             QObject::connect(job, &DeleteJob::finishedSignal, this, &PropagateUploadFileNG::slotDeleteJobFinished);
             _jobs.append(job);
             job->start();
@@ -239,7 +256,7 @@ void PropagateUploadFileNG::startNewUpload()
     }
     _transferId = uint(Utility::rand() ^ uint(_item->_modtime) ^ (uint(_fileToUpload._size) << 16) ^ qHash(_fileToUpload._file));
     _sent = 0;
-    _currentChunk = 0;
+    _currentChunk = 1; // Chunked upload v2: numbers range from 1 to 10000
 
     propagator()->reportProgress(*_item, 0);
 
@@ -259,7 +276,8 @@ void PropagateUploadFileNG::startNewUpload()
 
     // But we should send the temporary (or something) one.
     headers["OC-Total-Length"] = QByteArray::number(_fileToUpload._size);
-    auto job = new MkColJob(propagator()->account(), chunkUrl(), headers, this);
+    headers["Destination"] = destinationHeader();
+    const auto job = new MkColJob(propagator()->account(), chunkUploadFolderUrl(), headers, this);
 
     connect(job, &MkColJob::finishedWithError,
         this, &PropagateUploadFileNG::slotMkColFinished);
@@ -275,7 +293,7 @@ void PropagateUploadFileNG::slotMkColFinished()
     auto job = qobject_cast<MkColJob *>(sender());
     slotJobDestroyed(job); // remove it from the _jobs list
     QNetworkReply::NetworkError err = job->reply()->error();
-    _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toUInt();
 
     if (err != QNetworkReply::NoError || _item->_httpErrorCode != 201) {
         _item->_requestId = job->requestId();
@@ -287,52 +305,57 @@ void PropagateUploadFileNG::slotMkColFinished()
     startNextChunk();
 }
 
+void PropagateUploadFileNG::finishUpload()
+{
+    Q_ASSERT(_jobs.isEmpty()); // There should be no running job anymore
+    _finished = true;
+
+    // Finish with a MOVE
+    // If we changed the file name, we must store the changed filename in the remote folder, not the original one.
+    const auto destination = QDir::cleanPath(propagator()->account()->davUrl().path() + propagator()->fullRemotePath(_fileToUpload._file));
+    auto headers = PropagateUploadFileCommon::headers();
+
+    // "If-Match applies to the source, but we are interested in comparing the etag of the destination
+    const auto ifMatch = headers.take(QByteArrayLiteral("If-Match"));
+    if (!ifMatch.isEmpty()) {
+        headers[QByteArrayLiteral("If")] = "<" + QUrl::toPercentEncoding(destination, "/") + "> ([" + ifMatch + "])";
+    }
+
+    if (!_transmissionChecksumHeader.isEmpty()) {
+        qCInfo(lcPropagateUpload) << destination << _transmissionChecksumHeader;
+        headers[checkSumHeaderC] = _transmissionChecksumHeader;
+    }
+
+    const auto fileSize = _fileToUpload._size;
+    headers[QByteArrayLiteral("OC-Total-Length")] = QByteArray::number(fileSize);
+
+    const auto job = new MoveJob(propagator()->account(), Utility::concatUrlPath(chunkUploadFolderUrl(), "/.file"), destination, headers, this);
+    _jobs.append(job);
+    connect(job, &MoveJob::finishedSignal, this, &PropagateUploadFileNG::slotMoveJobFinished);
+    connect(job, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
+    propagator()->_activeJobList.append(this);
+    adjustLastJobTimeout(job, fileSize);
+    job->start();
+    return;
+}
+
 void PropagateUploadFileNG::startNextChunk()
 {
     if (propagator()->_abortRequested)
         return;
 
-    qint64 fileSize = _fileToUpload._size;
-    ENFORCE(fileSize >= _sent, "Sent data exceeds file size");
-
+    const auto fileSize = _fileToUpload._size;
+    ENFORCE(fileSize >= _sent, "Sent data exceeds file size")
     // prevent situation that chunk size is bigger then required one to send
     _currentChunkSize = qMin(propagator()->_chunkSize, fileSize - _sent);
 
     if (_currentChunkSize == 0) {
-        Q_ASSERT(_jobs.isEmpty()); // There should be no running job anymore
-        _finished = true;
-
-        // Finish with a MOVE
-        // If we changed the file name, we must store the changed filename in the remote folder, not the original one.
-        QString destination = QDir::cleanPath(propagator()->account()->davUrl().path()
-            + propagator()->fullRemotePath(_fileToUpload._file));
-        auto headers = PropagateUploadFileCommon::headers();
-
-        // "If-Match applies to the source, but we are interested in comparing the etag of the destination
-        auto ifMatch = headers.take(QByteArrayLiteral("If-Match"));
-        if (!ifMatch.isEmpty()) {
-            headers[QByteArrayLiteral("If")] = "<" + QUrl::toPercentEncoding(destination, "/") + "> ([" + ifMatch + "])";
-        }
-        if (!_transmissionChecksumHeader.isEmpty()) {
-            qCInfo(lcPropagateUpload) << destination << _transmissionChecksumHeader;
-            headers[checkSumHeaderC] = _transmissionChecksumHeader;
-        }
-        headers[QByteArrayLiteral("OC-Total-Length")] = QByteArray::number(fileSize);
-
-        auto job = new MoveJob(propagator()->account(), Utility::concatUrlPath(chunkUrl(), "/.file"),
-            destination, headers, this);
-        _jobs.append(job);
-        connect(job, &MoveJob::finishedSignal, this, &PropagateUploadFileNG::slotMoveJobFinished);
-        connect(job, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
-        propagator()->_activeJobList.append(this);
-        adjustLastJobTimeout(job, fileSize);
-        job->start();
+        finishUpload();
         return;
     }
 
-    const QString fileName = _fileToUpload._path;
-    auto device = std::make_unique<UploadDevice>(
-            fileName, _sent, _currentChunkSize, &propagator()->_bandwidthManager);
+    const auto fileName = _fileToUpload._path;
+    auto device = std::make_unique<UploadDevice>(fileName, _sent, _currentChunkSize, &propagator()->_bandwidthManager);
     if (!device->open(QIODevice::ReadOnly)) {
         qCWarning(lcPropagateUploadNG) << "Could not prepare upload device: " << device->errorString();
 
@@ -348,13 +371,14 @@ void PropagateUploadFileNG::startNextChunk()
 
     QMap<QByteArray, QByteArray> headers;
     headers["OC-Chunk-Offset"] = QByteArray::number(_sent);
+    headers["Destination"] = destinationHeader();
 
     _sent += _currentChunkSize;
-    QUrl url = chunkUrl(_currentChunk);
+    const auto url = chunkUrl(_currentChunk);
 
     // job takes ownership of device via a QScopedPointer. Job deletes itself when finishing
-    auto devicePtr = device.get(); // for connections later
-    auto *job = new PUTFileJob(propagator()->account(), url, std::move(device), headers, _currentChunk, this);
+    const auto devicePtr = device.get(); // for connections later
+    const auto job = new PUTFileJob(propagator()->account(), url, std::move(device), headers, _currentChunk, this);
     _jobs.append(job);
     connect(job, &PUTFileJob::finishedSignal, this, &PropagateUploadFileNG::slotPutFinished);
     connect(job, &PUTFileJob::uploadProgress,
@@ -410,10 +434,7 @@ void PropagateUploadFileNG::slotPutFinished()
         qint64 targetSize = propagator()->_chunkSize / 2 + predictedGoodSize / 2;
 
         // Adjust the dynamic chunk size _chunkSize used for sizing of the item's chunks to be send
-        propagator()->_chunkSize = qBound(
-            propagator()->syncOptions()._minChunkSize,
-            targetSize,
-            propagator()->syncOptions()._maxChunkSize);
+        propagator()->_chunkSize = ::qBound(propagator()->syncOptions().minChunkSize(), targetSize, propagator()->syncOptions().maxChunkSize());
 
         qCInfo(lcPropagateUploadNG) << "Chunked upload of" << _currentChunkSize << "bytes took" << uploadTime.count()
                                   << "ms, desired is" << targetDuration.count() << "ms, expected good chunk size is"
