@@ -102,12 +102,12 @@ private slots:
 
         LsColXMLParser parser;
 
-        connect( &parser, SIGNAL(directoryListingSubfolders(const QStringList&)),
-                 this, SLOT(slotDirectoryListingSubFolders(const QStringList&)) );
-        connect( &parser, SIGNAL(directoryListingIterated(const QString&, const QMap<QString,QString>&)),
-                 this, SLOT(slotDirectoryListingIterated(const QString&, const QMap<QString,QString>&)) );
-        connect( &parser, SIGNAL(finishedWithoutError()),
-                 this, SLOT(slotFinishedSuccessfully()) );
+        connect( &parser, &LsColXMLParser::directoryListingSubfolders,
+                 this, &TestXmlParse::slotDirectoryListingSubFolders );
+        connect( &parser, &LsColXMLParser::directoryListingIterated,
+                 this, &TestXmlParse::slotDirectoryListingIterated );
+        connect( &parser, &LsColXMLParser::finishedWithoutError,
+                 this, &TestXmlParse::slotFinishedSuccessfully );
 
         QHash <QString, ExtraFolderInfo> sizes;
         QVERIFY(parser.parse( testXml, &sizes, "/oc/remote.php/dav/sharefolder" ));
@@ -176,12 +176,12 @@ private slots:
 
         LsColXMLParser parser;
 
-        connect( &parser, SIGNAL(directoryListingSubfolders(const QStringList&)),
-                 this, SLOT(slotDirectoryListingSubFolders(const QStringList&)) );
-        connect( &parser, SIGNAL(directoryListingIterated(const QString&, const QMap<QString,QString>&)),
-                 this, SLOT(slotDirectoryListingIterated(const QString&, const QMap<QString,QString>&)) );
-        connect( &parser, SIGNAL(finishedWithoutError()),
-                 this, SLOT(slotFinishedSuccessfully()) );
+        connect( &parser, &LsColXMLParser::directoryListingSubfolders,
+                 this, &TestXmlParse::slotDirectoryListingSubFolders );
+        connect( &parser, &LsColXMLParser::directoryListingIterated,
+                 this, &TestXmlParse::slotDirectoryListingIterated );
+        connect( &parser, &LsColXMLParser::finishedWithoutError,
+                 this, &TestXmlParse::slotFinishedSuccessfully );
 
         QHash <QString, ExtraFolderInfo> sizes;
         QVERIFY(false == parser.parse( testXml, &sizes, "/oc/remote.php/dav/sharefolder" )); // verify false
@@ -199,12 +199,12 @@ private slots:
 
         LsColXMLParser parser;
 
-        connect( &parser, SIGNAL(directoryListingSubfolders(const QStringList&)),
-                 this, SLOT(slotDirectoryListingSubFolders(const QStringList&)) );
-        connect( &parser, SIGNAL(directoryListingIterated(const QString&, const QMap<QString,QString>&)),
-                 this, SLOT(slotDirectoryListingIterated(const QString&, const QMap<QString,QString>&)) );
-        connect( &parser, SIGNAL(finishedWithoutError()),
-                 this, SLOT(slotFinishedSuccessfully()) );
+        connect( &parser, &LsColXMLParser::directoryListingSubfolders,
+                 this, &TestXmlParse::slotDirectoryListingSubFolders );
+        connect( &parser, &LsColXMLParser::directoryListingIterated,
+                 this, &TestXmlParse::slotDirectoryListingIterated );
+        connect( &parser, &LsColXMLParser::finishedWithoutError,
+                 this, &TestXmlParse::slotFinishedSuccessfully );
 
         QHash <QString, ExtraFolderInfo> sizes;
         QVERIFY(false == parser.parse( testXml, &sizes, "/oc/remote.php/dav/sharefolder" )); // verify false
@@ -221,12 +221,12 @@ private slots:
 
         LsColXMLParser parser;
 
-        connect( &parser, SIGNAL(directoryListingSubfolders(const QStringList&)),
-                 this, SLOT(slotDirectoryListingSubFolders(const QStringList&)) );
-        connect( &parser, SIGNAL(directoryListingIterated(const QString&, const QMap<QString,QString>&)),
-                 this, SLOT(slotDirectoryListingIterated(const QString&, const QMap<QString,QString>&)) );
-        connect( &parser, SIGNAL(finishedWithoutError()),
-                 this, SLOT(slotFinishedSuccessfully()) );
+        connect( &parser, &LsColXMLParser::directoryListingSubfolders,
+                 this, &TestXmlParse::slotDirectoryListingSubFolders );
+        connect( &parser, &LsColXMLParser::directoryListingIterated,
+                 this, &TestXmlParse::slotDirectoryListingIterated );
+        connect( &parser, &LsColXMLParser::finishedWithoutError,
+                 this, &TestXmlParse::slotFinishedSuccessfully );
 
         QHash <QString, ExtraFolderInfo> sizes;
         QVERIFY(false == parser.parse( testXml, &sizes, "/oc/remote.php/dav/sharefolder" )); // verify false
@@ -260,12 +260,12 @@ private slots:
 
         LsColXMLParser parser;
 
-        connect( &parser, SIGNAL(directoryListingSubfolders(const QStringList&)),
-                 this, SLOT(slotDirectoryListingSubFolders(const QStringList&)) );
-        connect( &parser, SIGNAL(directoryListingIterated(const QString&, const QMap<QString,QString>&)),
-                 this, SLOT(slotDirectoryListingIterated(const QString&, const QMap<QString,QString>&)) );
-        connect( &parser, SIGNAL(finishedWithoutError()),
-                 this, SLOT(slotFinishedSuccessfully()) );
+        connect( &parser, &LsColXMLParser::directoryListingSubfolders,
+                 this, &TestXmlParse::slotDirectoryListingSubFolders );
+        connect( &parser, &LsColXMLParser::directoryListingIterated,
+                 this, &TestXmlParse::slotDirectoryListingIterated );
+        connect( &parser, &LsColXMLParser::finishedWithoutError,
+                 this, &TestXmlParse::slotFinishedSuccessfully );
 
         QHash <QString, ExtraFolderInfo> sizes;
         QVERIFY(!parser.parse( testXml, &sizes, "/oc/remote.php/dav/sharefolder" ));
@@ -325,12 +325,12 @@ private slots:
 
         LsColXMLParser parser;
 
-        connect( &parser, SIGNAL(directoryListingSubfolders(const QStringList&)),
-                 this, SLOT(slotDirectoryListingSubFolders(const QStringList&)) );
-        connect( &parser, SIGNAL(directoryListingIterated(const QString&, const QMap<QString,QString>&)),
-                 this, SLOT(slotDirectoryListingIterated(const QString&, const QMap<QString,QString>&)) );
-        connect( &parser, SIGNAL(finishedWithoutError()),
-                 this, SLOT(slotFinishedSuccessfully()) );
+        connect( &parser, &LsColXMLParser::directoryListingSubfolders,
+                 this, &TestXmlParse::slotDirectoryListingSubFolders );
+        connect( &parser, &LsColXMLParser::directoryListingIterated,
+                 this, &TestXmlParse::slotDirectoryListingIterated );
+        connect( &parser, &LsColXMLParser::finishedWithoutError,
+                 this, &TestXmlParse::slotFinishedSuccessfully );
 
         QHash <QString, ExtraFolderInfo> sizes;
         QVERIFY(false == parser.parse( testXml, &sizes, "/oc/remote.php/dav/sharefolder" ));
@@ -390,12 +390,12 @@ private slots:
 
         LsColXMLParser parser;
 
-        connect( &parser, SIGNAL(directoryListingSubfolders(const QStringList&)),
-                 this, SLOT(slotDirectoryListingSubFolders(const QStringList&)) );
-        connect( &parser, SIGNAL(directoryListingIterated(const QString&, const QMap<QString,QString>&)),
-                 this, SLOT(slotDirectoryListingIterated(const QString&, const QMap<QString,QString>&)) );
-        connect( &parser, SIGNAL(finishedWithoutError()),
-                 this, SLOT(slotFinishedSuccessfully()) );
+        connect( &parser, &LsColXMLParser::directoryListingSubfolders,
+                 this, &TestXmlParse::slotDirectoryListingSubFolders );
+        connect( &parser, &LsColXMLParser::directoryListingIterated,
+                 this, &TestXmlParse::slotDirectoryListingIterated );
+        connect( &parser, &LsColXMLParser::finishedWithoutError,
+                 this, &TestXmlParse::slotFinishedSuccessfully );
 
         QHash <QString, ExtraFolderInfo> sizes;
         QVERIFY(false == parser.parse( testXml, &sizes, "/oc/remote.php/dav/sharefolder" ));
@@ -455,12 +455,12 @@ private slots:
 
         LsColXMLParser parser;
 
-        connect( &parser, SIGNAL(directoryListingSubfolders(const QStringList&)),
-                 this, SLOT(slotDirectoryListingSubFolders(const QStringList&)) );
-        connect( &parser, SIGNAL(directoryListingIterated(const QString&, const QMap<QString,QString>&)),
-                 this, SLOT(slotDirectoryListingIterated(const QString&, const QMap<QString,QString>&)) );
-        connect( &parser, SIGNAL(finishedWithoutError()),
-                 this, SLOT(slotFinishedSuccessfully()) );
+        connect( &parser, &LsColXMLParser::directoryListingSubfolders,
+                 this, &TestXmlParse::slotDirectoryListingSubFolders );
+        connect( &parser, &LsColXMLParser::directoryListingIterated,
+                 this, &TestXmlParse::slotDirectoryListingIterated );
+        connect( &parser, &LsColXMLParser::finishedWithoutError,
+                 this, &TestXmlParse::slotFinishedSuccessfully );
 
         QHash <QString, ExtraFolderInfo> sizes;
         QVERIFY(parser.parse( testXml, &sizes, "/oc/remote.php/dav/sharefolder" ));
@@ -529,12 +529,12 @@ private slots:
 
         LsColXMLParser parser;
 
-        connect( &parser, SIGNAL(directoryListingSubfolders(const QStringList&)),
-                 this, SLOT(slotDirectoryListingSubFolders(const QStringList&)) );
-        connect( &parser, SIGNAL(directoryListingIterated(const QString&, const QMap<QString,QString>&)),
-                 this, SLOT(slotDirectoryListingIterated(const QString&, const QMap<QString,QString>&)) );
-        connect( &parser, SIGNAL(finishedWithoutError()),
-                 this, SLOT(slotFinishedSuccessfully()) );
+        connect( &parser, &LsColXMLParser::directoryListingSubfolders,
+                 this, &TestXmlParse::slotDirectoryListingSubFolders );
+        connect( &parser, &LsColXMLParser::directoryListingIterated,
+                 this, &TestXmlParse::slotDirectoryListingIterated );
+        connect( &parser, &LsColXMLParser::finishedWithoutError,
+                 this, &TestXmlParse::slotFinishedSuccessfully );
 
         QHash <QString, ExtraFolderInfo> sizes;
         QVERIFY(!parser.parse( testXml, &sizes, "/oc/remote.php/dav/sharefolder" ));
@@ -594,12 +594,12 @@ private slots:
 
         LsColXMLParser parser;
 
-        connect( &parser, SIGNAL(directoryListingSubfolders(const QStringList&)),
-                 this, SLOT(slotDirectoryListingSubFolders(const QStringList&)) );
-        connect( &parser, SIGNAL(directoryListingIterated(const QString&, const QMap<QString,QString>&)),
-                 this, SLOT(slotDirectoryListingIterated(const QString&, const QMap<QString,QString>&)) );
-        connect( &parser, SIGNAL(finishedWithoutError()),
-                 this, SLOT(slotFinishedSuccessfully()) );
+        connect( &parser, &LsColXMLParser::directoryListingSubfolders,
+                 this, &TestXmlParse::slotDirectoryListingSubFolders );
+        connect( &parser, &LsColXMLParser::directoryListingIterated,
+                 this, &TestXmlParse::slotDirectoryListingIterated );
+        connect( &parser, &LsColXMLParser::finishedWithoutError,
+                 this, &TestXmlParse::slotFinishedSuccessfully );
 
         QHash <QString, ExtraFolderInfo> sizes;
         QVERIFY(parser.parse( testXml, &sizes, QString::fromUtf8("/ä") ));
